import macondo.mjada.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

/**
 *	A simple chat application build upon Macondo coordination features.
 */

public class ChatClient implements ActionListener, Runnable {
	private Space 	space;
	private boolean	flag = true;
	private String	userName;
	// GUI stuff
	Frame		frame;
	TextArea	textarea;
	TextField	msg_tf;
	TextField	user_name_tf;
	Button		button;
	int			width = 400,
				height = 400;
	
	public static void main(String[] args) {
		ChatClient client = new ChatClient();
		client.parseCmdLine(args);
		client.initGUI();
		new Thread(client).start();
	}
	
	public void parseCmdLine(String[] args) {
		String	host = "localhost";
		int		port = MJada.DEFAULT_PORT;
		
		for(int i=0;i<args.length;i++) {
			if (args[i].equals("-r")) {
				if (i+1 < args.length) {
					++i;
					host = args[i];
				} else {
					System.out.println("Wrong number of parameters. Exiting.");
					System.exit(0);
				}
				
				if (i+1 < args.length) {
					++i;
					try {
						port = Integer.parseInt(args[i]);
					} catch (NumberFormatException nfe) {
						port = MJada.DEFAULT_PORT;
					}
				}

			} else if (args[i].equals("-h") || args[i].equals("-help")) {
				System.out.println("Usage: java ChatClient [-r [hostname [port]]]");
				System.exit(0);
			}
		}
		
		space = new SpaceClient(host, port);
	}
	
	public void initGUI() {
		frame = new Frame("Chat");
		frame.setLayout(null);
		
		textarea = new TextArea(80,20);
		textarea.setEditable(false);
		
		msg_tf = new TextField(80);
		user_name_tf = new TextField(8);
		button = new Button("Submit");
		button.addActionListener(this);
		frame.setSize(width,height);
		
		frame.add(textarea);
		textarea.setBounds(5,25,380,200);
		frame.add(msg_tf);
		msg_tf.setBounds(10,240,240,30);
		frame.add(user_name_tf);
		user_name_tf.setBounds(10,320,80,30);
		frame.add(button);
		button.setBounds(270,240,100,30);
		
		frame.pack();
		frame.show();
	}
	
	public void run() {
		messageLoop();
	}
	
	public void messageLoop() {
		int msg_index = 0;
		Tuple 	formal = new Tuple(String.class,new Integer(msg_index),String.class,String.class),
				result, tmp;
		Result	res;
		String	type, from, msg;
		
		userName = user_name_tf.getText();
		Object tmp_obj;
		
		while(flag) {
			res = space.read(formal);
			tmp_obj = res.getObject();
			System.out.println("class is "+tmp_obj.getClass());
			result = (Tuple)tmp_obj;
			type = (String)result.getItem(0);
			from = (String)result.getItem(2);
			msg = (String)result.getItem(3);
			
			if ( type.equals("_msg") ) {
				textarea.append(from+":"+space.getSpaceName()+"> " + msg + "\n");
				++msg_index;
			} else if ( type.equals("_join") && from.equals(userName) ) {
				res = space.in(result);
				res.getObject();
				space.join( msg );
				
				res = space.read( new Tuple("_index", Integer.class) );
				if ( !(res.isAvailable()) ) {
					space.out( new Tuple("_index", new Integer(0)) );
				}
				res.kill();
				msg_index = 0;
			
			} else if ( type.equals("_leave") && from.equals(userName) ) {
				res = space.in(result);
				res.getObject();
				space.leave();
				msg_index = 0;
			} else {
				++msg_index;
			}
			
			formal.setItem(1,new Integer(msg_index));
		}
	}
	
	public void actionPerformed(ActionEvent e) {
		String	source = e.getActionCommand(),
				msg = msg_tf.getText();
		
		msg_tf.setText("");
		
		if (userName.equals("")) {
			userName = user_name_tf.getText();
		}
		
		int index;
		Result res = space.in(new Tuple("_index",Integer.class));
		Tuple tuple;
		if ( (tuple = (Tuple)res.getObject(1000)) != null ) {
			index = ((Integer)tuple.getItem(1)).intValue();
		} else {
			// space.out(new Tuple("_index", new Integer(0)));
			index = 0;
			res.kill();
		}
		
		if (source.equals("Submit")) {
			if (msg.startsWith("join")) {
				space.out( new Tuple("_join", new Integer(index++),
					user_name_tf.getText(), msg.substring(5)));
			} else if (msg.equals("leave")) {
				space.out( new Tuple("_leave", new Integer(index++),
					user_name_tf.getText(), new String()));
			} else { // normal message handling
				space.out(new Tuple("_msg", new Integer(index++),
					user_name_tf.getText(), msg));
			}
		}
		
		space.out(new Tuple("_index", new Integer(index)));
	}
	
	
}