package macondo.mak;

import java.util.Vector;
import java.util.Enumeration;

import macondo.util.*;

/**
 *
 *	Class used to encapsulate the itinerary of an agent.
 *	An itinerary is a sequence of <code>Pair<code>s, where a <code>Pair<code>
 *	is an association destination/task and each task is executed at the
 *	corresponding location.
 *
 *	@see		Pair
 *
 *	@author 	Andrea Giovannini
 *	@version	beta
 */


public class Itinerary implements java.io.Serializable {
	private Vector itinerary;
	
	// Constructors ------------------------------------------------------
	
	public Itinerary() {
		itinerary = new Vector();
	}
	
	public Itinerary(Vector itinerary) {
		itinerary = itinerary;
	}
	
	// Public Methods ----------------------------------------------------
	
	public void setItinerary(Vector itinerary) {
		itinerary = itinerary;
	}
	
	public void add(Pair p) {
		itinerary.addElement(p);
	}
	
	public void add(PlaceName pn, String methodName) {
		itinerary.addElement(new Pair(pn, methodName));
	}
	
	public void add(String host, int port, String methodName) {
		itinerary.addElement(new Pair(new PlaceName(host, port), methodName));
	}
	
	public Pair drop() {
		Pair tmp = (Pair)itinerary.firstElement();
		itinerary.removeElement(tmp);;
		return tmp;
	}
	
	public boolean hasMoreElements() {
		return ( itinerary.size() > 0 );
	}
	
	public void modifyAt(Pair p, int index) {
		itinerary.setElementAt(p,index);
	}
	
}

