package macondo.mak;


/**
 *	<code>Traveller<code> is a mobile agent following an itinerary
 *
 *	@see	Itinerary
 *	@see	Agent
 *
 *	@author		Andrea Giovannini
 *	@version	beta
 */

public class Traveller extends Agent {
	private Itinerary	myItinerary;
	
	private PlaceName	placeToReach;
	private String		methodToExecute;
	
	// Constructors ------------------------------------------------------
	
	/**
	 *	Creates a new <code>Traveller</code>.
	 */
	protected Traveller() {}

	/**
	 *	Creates a new <code>Traveller</code> with a given name.
	 *
	 *	@param	name	the <code>Traveller</code> name.
	 */
	protected Traveller(String name) {
		super(name);
	}
	
	/**
	 *	Creates a new <code>Traveller</code> with a given itinerary.
	 *
	 *	@param	itinerary	the <code>Traveller</code> itinerary.
	 *	@see	Itinerary
	 */
	protected Traveller(Itinerary itinerary) {
		myItinerary = itinerary;
	}
	
	/**
	 *	Creates a new <code>Traveller</code> with given name
	 *	and itinerary.
	 *
	 *	@param	name		the <code>Traveller</code> name.
	 *	@param	itinerary	the <code>Traveller</code> itinerary.
	 *	@see	Itinerary
	 */
	protected Traveller(String name, Itinerary itinerary) {
		super(name);
		myItinerary = itinerary;
	}
	
	// Public Methods ----------------------------------------------------

	/**
	 *	Sets itinerary.
	 *
	 *	@param	itinerary	the itinerary.
	 */
	public final void setItinerary(Itinerary itinerary) {
		myItinerary = itinerary;
	}
	
	/**
	 *	Let the traveller move, following its itinerary.
	 *
	 *	@see	Pair
	 */
	public final void doTrip() {
		Pair		pair;
		
		if ( myItinerary.hasMoreElements() ) {
			pair = myItinerary.drop();
			placeToReach = pair.getPlace();
			methodToExecute = pair.getMethodName();
			
			try {
				go( placeToReach, "executeCurrentMethod" );
			} catch (Exception e) {
				error(e);
			}
		}
	}
	
	// Protected Methods -------------------------------------------------
	
	/**
	 *	Method called when a problem is encountered during a trip.
	 *	This default implementation does nothing.
	 */
	public void error(Exception e) {
		//go( getSource(), "displayInfo" );
	}
	
	// Package Methods ---------------------------------------------------
	
	/**
	 *	Executes the method associated to the current place, then
	 *	continues trip.
	 */
	final void executeCurrentMethod() {
		try {
			executeMethod( methodToExecute , null , null);
		} catch (Exception e) {
			error(e);
		}
		doTrip();
	}
}