package macondo.mjada;

import java.io.*;
import jada.*;

/**
 *	This is the global serializer. Using Java object
 *	serialization it can serialize and deserialize every Java object
 *	that implements the java.io.Serializable interface.
 *
 *	@author		Andrea Giovannini
 *	@version	beta
 */

public class GlobalSerializer implements JadaSerializer {

	// Constructor -------------------------------------------------------
	/**
	 *	Default constructor.
	 */
	public GlobalSerializer() {}
	
	// Public Methods ----------------------------------------------------
	
	/**
	 *	JadaSerializer interface method.
	 *	Dumps the given object into the specified output stream.
	 *
	 *	@param	object	the object to dump.
	 *	@param	os		the output stream where the object is written.
	 *	@exception		IOException		thrown if any I/O error occurs.
	 *	@exception		ItemNotStreamableException	thrown if the
	 *					object is not streamable.
	 */
	public void dump(Object object, OutputStream os) throws IOException,
			ItemNotStreamableException {
		try {
			ObjectOutputStream oos = new ObjectOutputStream(os);
			oos.writeObject(object);
		} catch (JadaItemIsNotStreamableException e) {
			throw new ItemNotStreamableException();
		}
	}
	
	/**
	 *	JadaSerializer interface method.
	 *	Returns an object read from a stream written by another serializer.
	 *
	 *	@param	is	the input stream from which the object is read.
	 *	@return		an object read from the input stream.
	 *	@exception	SpaceException	thrown if an error occurs while
	 *								performing an operation on a tuple space.
	 */
	public Object restore(InputStream is) throws SpaceException {
		Object ret = null;
		try {
			ObjectInputStream ois = new ObjectInputStream(is);
			ret = ois.readObject();
		} catch (JadaIOException jioe) {
			throw new SpaceException();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return ret;
	}
	
}