package macondo.mjada;

import java.io.Serializable;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;
import jada.JadaItem;

/**
 *	Any class used in a tuple must extend this class. Note that
 *	- a class that extends <code>Item</code> must have a default constructor
 *	- a class that extends <code>Item</code> must be public
 */

public abstract class Item implements Serializable, JadaItem {

	// Public Methods ----------------------------------------------------

	// Abstract Methods

	/**
	 *	Dumps this object into the specified output stream.
	 *
	 *	@param	os	the output stream.
	 *	@exception	ItemNotStreamableException	thrown if the item
	 *						is not streamable.
	 *	@exception	IOException					thrown if any I/O
	 *						error occurs.
	 */
	public abstract void dumpItem(OutputStream os) throws
		ItemNotStreamableException, IOException;
	
	/**
	 *	Restore the object from the specified input stream.
	 *
	 *	@param	is	the input stream.
	 *	@exception	ItemNotStreamableException	thrown if the item
	 *						is not streamable.
	 *	@exception	IOException					thrown if any I/O
	 *						error occurs.
	 */
	public abstract void buildItem(InputStream is) throws
		ItemNotStreamableException, IOException;
	
	/**
	 *	Clone this item from another one.
	 *
	 *	@param	source	the object to clone.
	 */
	public abstract void cloneItem(Object source);
	
	/**
	 *	Tests if this item matches the given one.
	 *
	 *	@param	match	the item to match.
	 */
	public abstract boolean matchesItem(Item match);
	
	// Concrete Methods
	
	/**
	 *	This is the JadaItem interface method used for matching. Since
	 *	it requires a JadaItem argument it needs to be hidden from
	 *	the MJada API. So it simply checks if the argument is an
	 *	<code>Item</code> object and then it invokes the MJada
	 *	<code>matchesItem()</code> methods.
	 *
	 *	@param	match	the item to match.
	 */
	public final boolean matchesItem(JadaItem match) {
		if ( match instanceof Item) {
			return matchesItem( (Item)match );
		} else {
			return false;
		}
	}
	
}