package macondo.mjada;

import jada.*;
import jada.net.*;

/**
 *	A <code>SpaceClient</code> is an interface to a remote space server.
 *
 *	@author		Andrea Giovannini
 *	@version	beta
 */

public class SpaceClient extends Space {

	// Constructors ------------------------------------------------------
	
	/**
	 *	Default constructor.
	 *	Builds a new space client to connect a space server
	 *	on localhost.
	 */
	public SpaceClient() {
		this("localhost", JadaNetConst.DEFAULT_PORT);
	}
	
	/**
	 *	Builds a new space client to connect a space server
	 *	with specified host name and port.
	 *
	 *	@param	host	the host name.
	 *	@param	port	the port number.
	 */
	public SpaceClient(String host, int port) {
		setObjectSpace( new ObjectClient(host,port) );
		initSpaceName();
	}
	
	/**
	 *	Builds a new space client to connect a space server
	 *	with specified host name at default port.
	 *
	 *	@param	host	the host name.
	 */
	public SpaceClient(String host) {
		this(host, JadaNetConst.DEFAULT_PORT);
	}
	
	/**
	 *	Builds a new space client to connect a space server
	 *	on host localhost at the specified port number.
	 *
	 *	@param	port	the port number.
	 */
	public SpaceClient(int port) {
		this("localhost",port);
	}
	
}