package macondo.mjada;

import jada.net.ObjectServer;

/**
 *	This class implements a remote space server. It is just a wrapper
 *	to the <code>jada.net.ObjectServer</code> class.
 *
 *	@author		Andrea Giovannini
 *	@version	beta
 */

public class SpaceServer extends ObjectServer {

	// Constructors ------------------------------------------------------
	
	/**
	 *	Creates a new <code>SpaceServer</code> object listening at the
	 *	default port.
	 */
	public SpaceServer() {
		super();
	}
	
	/**
	 *	Creates a new <code>SpaceServer</code> object listening at the
	 *	specified port.
	 *
	 *	@param	port	the port used to accept network connections.
	 */
	public SpaceServer(int port) {
		super(port);
	}

	/**
	 *	Main entry point when used as a standalone Java program.
	 */
	public static void main(String[] args) {
		SpaceServer server;
		int port = DEFAULT_PORT;
		boolean verbose = false;
		
		// parse cmd line switches
		for(int i=0; i < args.length; i++) {
			if (args[i].equals("-p")) {
				try {
					port = Integer.parseInt(args[++i]);
				} catch (NumberFormatException e) {
					System.out.println("Bad port number, default used.");
				}
			} else if (args[i].equals("-v")) {
				verbose = true;
			} else if (args[i].equals("-h") || args[i].equals("-help")) {
				System.out.println("Usage: java SpaceServer [-p port] [-v] [-h[elp]]");
				System.exit(0);
			} else if (args[i].startsWith("-")) {
				System.out.println("Unknown option.");
				System.exit(-1);
			}
		}
		
		// set up the server
		server = new SpaceServer(port);
		server.setVerbose(verbose);
		new Thread(server).start();

		server.waitToStart();
		if (verbose) {
			System.out.println("SpaceServer running at port #" + port);
		}
	}
	
}