package macondo.mjada.collections;

import macondo.mjada.*;
import jada.ObjectSpace;

/**
 *	A <code>ReadIterator</code> is an iterator that reads a sequence
 *	of tuples in a collection. If the collection is empty or all tuples
 *	have been read this iterator blocks, waiting for the insertion of
 *	other tuples.
 *
 *	@author		Andrea Giovannini
 *	@version	beta
 */

public class ReadIterator implements TupleIterator {

	private ObjectSpace		objSpace;
	private SpaceName		spaceName;
	private Token			currToken;
	private Tuple			pattern;
	
	// Constructor -------------------------------------------------------
	
	/**
	 *	Builds a new <code>ReadIterator</code> on the given space
	 *	to read tuples with the given pattern.
	 *
	 *	@param	space	the tuple space where collected tuples reside.
	 *	@param	pattern	the template of collected tuples.
	 */
	public ReadIterator(Space space, Tuple pattern) {
		this.objSpace = space.getObjectSpace();
		this.spaceName = space.getSpaceName();
		this.currToken = new Token();
		this.pattern = pattern;
	}
	
	// Public Methods ----------------------------------------------------
	
	/**
	 *	TupleIterator interface method.
	 *	Returns the next tuple in this iteration. If no tuple is available
	 *	blocks waiting for a corresponding insertion.
	 *
	 *	@return		the next tuple in this iteration.
	 */
	public Tuple nextTuple() {
		Tuple	tmp = null,
				result = null;
		
		tmp = (Tuple)objSpace.read( new Tuple(spaceName, currToken, pattern) );
		currToken.advance();
		result = (Tuple)tmp.getItem(2);
		return result;
	}
	
}
